from typing import Dict, Final
from minimalmodbus import NoResponseError
from .generic import ModbusRTUDevice


class SensorWiredIAQ(ModbusRTUDevice):
    '''
    Wired sensor measuring temperature, relative humidity,
    carbon dioxide and VOC, optionally particulate matter
    '''
    IDENTIFIER: int = 0xbeef
    input_registers: Dict[str, int] = {
            'T': 30010,    # from SHT4x
            'T_F': 30011,
            'RH': 30012,   # from SHT4x
            'CO2': 30013,  # from SCD4x
            'VOC_index': 30014,
            'VOC_ticks': 30015,
            'NOx_index': 30016,
            'NOx_ticks': 30017,
            'PM_mass_concentration_1.0': 30018,
            'PM_mass_concentration_2.5': 30019,
            'PM_mass_concentration_4.0': 30020,
            'PM_mass_concentration_10.0': 30021,
            'PM_number_concentration_0.5': 30022,
            'PM_number_concentration_1.0': 30023,
            'PM_number_concentration_2.5': 30024,
            'PM_number_concentration_4.0': 30025,
            'PM_number_concentration_10.0': 30026,
            'PM_typical_particle_size':  30027,
            'T_SCD4x': 30028,
            'T_SCF4x_F': 30029,
            'RH_SCD4x': 30030
            } | ModbusRTUDevice.input_registers
    # TODO use super, but __class__ not defined
    holding_registers: Dict[str, int] = {
            'LED_on': 40001,
            'LED_brightness': 40002,
            'LED_smooth': 40003,
            'CO2_alert_limit_1': 40004,
            'CO2_alert_limit_2': 40005,
            'SCD4x_temperature_offset': 40006,
            'MODBUS_address': 40007,
            'baudrate': 40008
            } | ModbusRTUDevice.holding_registers
    RESET_MAGIC_NUMBER: Final[int] = 0xABCD

    @property
    def CO2(self):
        return int(self.read_register(self.input_registers['CO2']))

    @property
    def T(self):
        # TODO maybe use signed version?
        return self.read_register(self.input_registers['T'],
                                  signed=True
                                  ) / 10

    @property
    def RH(self):
        return self.read_register(self.input_registers['RH'])
